package gen

import (
	"math/rand"
	"image"
)

type CellTypeId string


type CellType struct {
	Id CellTypeId
	Neighs []CellTypeId
	Color Rgba
}

func (typ *CellType) GetNeigh() CellTypeId {
	n := rand.Intn(len(typ.Neighs))
	ret := typ.Neighs[n]
	return ret
}

type Cell struct {
	Type CellTypeId
}

type Map struct {
	Size int
	Cells []Cell
	Types map[CellTypeId] *CellType
}

type Neigh struct {
	TypeId CellTypeId
	Chance int
}

func N(typeId CellTypeId, chance int) Neigh {
	return Neigh{
		TypeId: typeId,
		Chance: chance,
	}
}

func NewMap(size int) *Map {
	ret := &Map{
		Size: size,
		Cells: make([]Cell, size*size),
		Types: make(map[CellTypeId] *CellType),
	}
	return ret
}

func (m *Map) Def(id CellTypeId, color Rgba, neighs ...Neigh) *Map {

	ts := []CellTypeId{}
	for _, neigh := range neighs {
		for j := 0 ; j<neigh.Chance ; j++ {
			ts = append(ts, neigh.TypeId)
		}
	}

	m.Types[id] = &CellType{
		Id: id,
		Neighs: ts,
		Color: color,
	}

	return m
}

func (m *Map) FillFromUpLeft(typeId CellTypeId) {
	m.Cells[0] = Cell{
		Type: typeId,
	}
	for n := 1 ; n<m.Size ; n++ {
		for k := 0 ; k<n ; k++ {
			prevType := m.Types[
				m.Cells[n + m.Size*k - 1].Type,
			]
			m.Cells[n + m.Size*k] = Cell{
				Type: prevType.GetNeigh(),
			}

			prevType = m.Types[
				m.Cells[k + m.Size*(n-1)].Type,
			]
			m.Cells[k + m.Size*n] = Cell{
				Type: prevType.GetNeigh(),
			}
		}

		
		prevType := m.Types[
			m.Cells[
				n - 1 + m.Size*(n-1),
			].Type,
		]
		m.Cells[n + m.Size*(n)] = Cell{
			Type: prevType.GetNeigh(),
		}
	}
}

type Rgba struct {
	R, G, B, A float64
}

func (c Rgba) RGBA() (r, g, b, a uint32) {
	max := ^uint32(0)
	return uint32(c.R*float64(max)),
		uint32(c.G*float64(max)),
		uint32(c.B*float64(max)),
		uint32(c.A*float64(max))
}

func (m *Map) Render() image.Image {
	img := image.NewRGBA(image.Rectangle{
		image.Point{0, 0},
		image.Point{m.Size, m.Size},
	})
	for x := 0 ; x<m.Size ; x++ {
		for y := 0 ; y<m.Size ; y++ {
			cell := m.Cells[y+ x*m.Size]
			typ, ok := m.Types[cell.Type]
			if !ok {
				img.Set(x, y, Rgba{0, 0, 0, 0})
				continue
			}
			img.Set(x, y, typ.Color)
		}
	}
	return img
}















